/* Робототехнический проект: "ИСКАТЕЛЬ":
   - Двухзвенное шасси "Скрипыш" (Skripysh) 8х8 c роликовым межколёсным приводом,
     балансирной подвеской и ломающейся рамой, на дистанционном управлении со смартфона,
     оборудованное датчиками цвета и расстояния для обнаружения и распознавания заданных
     объектов сбора и автономным манипулятором с электромагнитным захватом для погрузки,
     удержания и транспортировки распознанного груза на базу для последующей выгрузки и
     складирования.
   Автор: Захаров Михаил
   Учреждение: ГБОУ лицей №226, класс 6Б
               ГБУ ДО ДДЮТ Фрунзенского района Санкт-Петербурга
   2023 год
*/

#define REMOTEXY_MODE__HARDSERIAL      // Определение режима соединения и подключение библиотеки RemoteXY
#include <RemoteXY.h>

#define REMOTEXY_SERIAL Serial         // Настройки соединения
#define REMOTEXY_SERIAL_SPEED 9600

#pragma pack(push, 1)                  // Конфигурация интерфейса пульта управления движением шасси на экране смартфона
uint8_t RemoteXY_CONF[] =
  { 255,4,0,0,0,43,0,10,13,0,
  4,176,-6,46,71,14,2,26,1,1,
  82,2,16,20,2,31,226,134,145,0,
  1,1,82,41,16,20,2,31,226,134,
  147,0,3,131,11,9,32,12,2,26 };

struct {                               // Структура определяет все переменные и события интерфейса управления
  int8_t rudder;                      // Руль: 100 - -100 положение слайдера
  uint8_t forward;                     // Вперёд: 1 если кнопка нажата, иначе 0
  uint8_t back;                        // Назад: 1 если кнопка нажата, иначе 0
  uint8_t speed;                       // Скорость: если переключатель в положении A - 0, B - 1, C - 2
  
  uint8_t connect_flag;                // 1 если есть подключение, иначе 0
  
}

RemoteXY;
#pragma pack(pop)

#define PIN_FORWARD 2                  // Назначаем пины подключения драйвера моторов L298N к Arduino 
#define PIN_BACK 4
#define PIN_SPEED 3

#include <Servo.h>                     //  Подключаем библиотеку для работы с сервоприводами
Servo servoRudder;                     //  Объявляем переменную руля шасси
int deg;                               //  Переменная угла поворота руля шасси

Servo servo1;                          //  Объявляем переменную Серво1 нижнего шарнира стрелы манипулятора
int angle1 = 125;                      //  Переменная для угла поворота Серво1 и её начальное значение
Servo servo2;                          //  Объявляем переменную Серво2 среднего шарнира стрелы манипулятора
int angle2 = 170;                      //  Переменная для угла поворота Серво2 и её начальное значение
Servo servo3;                          //  Объявляем переменную Серво3 шарнира магнитного захвата манипулятора
int angle3 = 140;                      //  Переменная для угла поворота Серво3 и её начальное значение

#include <NewPing.h>  // подключение библиотеки NewPing
#define TRIG_PIN A2 // назначаем порты подключения датчика
#define ECHO_PIN A3
#define MAX_DISTANCE 400 // Константа для определения максимального расстояния, которое мы будем считать корректным.
NewPing sonar(TRIG_PIN, ECHO_PIN, MAX_DISTANCE);  // создаем объект датчика расстояния

#include <Adafruit_TCS34725.h>         // Подключаем библиотеку для работы с датчиком цвета
// Создаём объект датчика цвета и указываем настройки
Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X); 

#define Led1 13                        //  Назначаем пин красного светодиода
#define Led2 12                        //  Назначаем пин синего светодиода
#define Magnet 11                      //  Назначаем пин электромагнита

bool cargoflag = false;                // Объявляем флаг для управления работой электромагнита

void setup() {
  RemoteXY_Init ();

  pinMode (PIN_FORWARD, OUTPUT);       //  Переводим пины управления движением шасси в режим вывода
  pinMode (PIN_BACK, OUTPUT);
  pinMode (PIN_SPEED, OUTPUT);

  servoRudder.attach(5);               //  Привязываем серво-мотор руля шасси к пину 5 
  RemoteXY.rudder = 0;                 //  Задаем начальное значение положение слайдера руля 

  pinMode(Led1, OUTPUT );              //  Переводим пин красного светодиода в режим вывода
  pinMode(Led2, OUTPUT );              //  Переводим пин синего светодиода в режим вывода
  pinMode(Magnet, OUTPUT );            //  Переводим пин электромагнита в режим вывода

  tcs.begin();                         //  Активируем датчик цвета
  
  servo1.attach(8);                    //  Привязываем Серво1 к пину 8
  servo1.write(angle1);                //  Задаем начальное положение угла поворота Серво1
  servo2.attach(9);                    //  Привязываем Серво2 к пину 9
  servo2.write(angle2);                //  Задаем начальное положение угла поворота Серво2
  servo3.attach(10);                   //  Привязываем Серво3 к пину 10
  servo3.write(angle3);                //  Задаем начальное положение угла поворота Серво3
}

void loop() {
  RemoteXY_Handler ();
  
  if (RemoteXY.speed == 0) {           // Управление скоростью движения шасси
    analogWrite (PIN_SPEED, 150);
  }
  if (RemoteXY.speed == 1) {
    analogWrite (PIN_SPEED, 200);
  }
  if (RemoteXY.speed == 2) {
    analogWrite (PIN_SPEED, 250);
  }

  digitalWrite(PIN_FORWARD, (RemoteXY.forward == 0) ? LOW : HIGH);    // Движение шасси вперед и назад 
  digitalWrite(PIN_BACK, (RemoteXY.back == 0) ? LOW : HIGH);

  deg = RemoteXY.rudder;               // Управление рулём шасси
  deg = map (deg, 100, -100, 135, 35); //  135 и 35 - крайние правое и левое положения 
  servoRudder.write (deg);             // поворачиваем сервопривод поворота шасси

  unsigned int dist = sonar.ping_cm();  // получаем значение от датчика расстояния и сохраняем его в переменную
  if (dist == 0) {dist = 400;}         // условие, когда расстояние не считывается, принимаем его за максимум
  float red, green, blue;              //  Создаём переменные, в которые записываются цвета объекта
  tcs.getRGB(&red, &green, &blue);     //  Запрашиваем у датчика цветовые компоненты

  if(red >= 105)           //  Условие, когда датчики четко распознают объект красного цвета...
  {
   if(dist <= 2 && cargoflag == false)      // ...на расстоянии 5 см от шасси (2 см от датчика)
   {
    cargoflag = true;                   //  Поднимаем флаг
    digitalWrite (Led1, HIGH);          //  Загорается красный светодиод – сигнал распознавания груза
    digitalWrite(PIN_FORWARD, LOW);     //  Движение шасси прекращается на время погрузки груза
    digitalWrite(PIN_BACK, LOW);

    // Подаём электромагнитный захват манипулятором к грузу перед шасси
    while (angle2 > 90)                 //  Управляем сервоприводом среднего шарнира манипулятора
    {
     angle2--;   //  Уменьшаем угол поворота Серво2 на 1 град.
     servo2.write(angle2);   //  Сообщаем Серво2 угол поворота
     delay(20);   //  Ждем 20 мс, пока Серво2 не сделает поворот
    }
    while (angle3 > 160)                //  Управляем сервоприводом магнитного захвата
    {
     angle3--;    //  Уменьшаем угол поворота Серво3 на 1 град.
     servo3.write(angle3);    //  Сообщаем Серво3 угол поворота
     delay(20);    //  Ждем 20 мс, пока Серво3 не сделает поворот
    }
    while (angle1 < 180)                //  Управляем сервоприводом нижнего шарнира манипулятора
    {
     angle1++;    //  Увеличиваем угол поворота Серво1 на 1 град.
     servo1.write(angle1);    //  Сообщаем Серво1 угол поворота
     delay(20);   //  Ждем 20 мс, пока Серво1 не сделает поворот
    }
    digitalWrite (Magnet, HIGH);       //  Включаем электромагнит
    delay(2000);                       //  Выдерживаем паузу для захвата груза электромагнитом

    // Перемещаем захват с грузом манипулятором на заднюю секцию шасси для транспортировки
    while (angle1 > 0)                  //  Управляем сервоприводом нижнего шарнира манипулятора
    {
     angle1--;    //  Увеличиваем угол поворота Серво1 на 1 град.
     servo1.write(angle1);    //  Сообщаем Серво1 угол поворота
     delay(20);    //  Ждем 20 мс, пока Серво1 не сделает поворот
    }
    while (angle3 > 105)                 //  Управляем сервоприводом магнитного захвата
    {
     angle3--;     //  Уменьшаем угол поворота Серво3 на 1 град.
     servo3.write(angle3);    //  Сообщаем Серво3 угол поворота
     delay(20);    //  Ждем 20 мс, пока Серво3 не сделает поворот
    }
    while (angle2 > 20)                  //   Управляем сервоприводом среднего шарнира манипулятора
    {
     angle2--;     //   Уменьшаем угол поворота Серво2 на 1 град.
     servo2.write(angle2);    //  Сообщаем Серво2 угол поворота
     delay(20);    //  Ждем 20 мс, пока Серво2 не сделает поворот
    }
   }
  } if(blue > 110 && cargoflag == true)  // Условие, когда датчик четко распознает сигнал подъемника синего цвета
  { 
   cargoflag = false;                  //  Опускаем флаг
   digitalWrite (Led2, HIGH);          //  Загорается синий светодиод – сигнал распознавания подъемника
   digitalWrite(PIN_FORWARD, LOW);     //  Движение шасси прекращается на время разгрузки шасси... 
   digitalWrite(PIN_BACK, LOW);        //  ... и возврата манипулятора в исходное положение

   while (angle3 > 95)                 //  Вырвниваем положение магнитного захвата с грузом
   {
    angle3--;   //  Уменьшаем угол поворота Серво3 на 1 град.
    servo3.write(angle3);    //  Сообщаем Серво3 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво3 не сделает поворот
   }
   while (angle2 > 10)                 //   Опускаем захват с грузом к платформе шасси
   {
    angle2--;     //   Уменьшаем угол поворота Серво2 на 1 град.
    servo2.write(angle2);    //  Сообщаем Серво2 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво2 не сделает поворот
   }
   delay(7000);                        //  Выдерживаем паузу для фиксации груза выдвижным захватом подъемника
   digitalWrite (Led1, LOW);           //  Красный светодиод гаснет
   digitalWrite (Magnet, LOW);         //  Электромагнит выключается
   delay(7000);                        //  Выдерживаем паузу для забора груза выдвижным захватом подъемника
  
   // Перемещаем захват в исходное положение для поиска нового груза
   while (angle1 < 20)                 //  Управляем сервоприводом нижнего шарнира манипулятора
   {
    angle1++;    //  Уменьшаем угол поворота Серво1 на 1 град.
    servo1.write(angle1);    //  Сообщаем Серво1 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво1 не сделает поворот
   }
   while (angle3 < 180)                //  Управляем сервоприводом шарнира магнитного захвата
   {
    angle3++;   //  Уменьшаем угол поворота Серво3 на 1 град.
    servo3.write(angle3);    //  Сообщаем Серво3 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво3 не сделает поворот
   }
   while (angle2 > 0)                  //   Управляем сервоприводом среднего шарнира манипулятора
   {
    angle2--;     //   Уменьшаем угол поворота Серво2 на 1 град.
    servo2.write(angle2);    //  Сообщаем Серво2 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво2 не сделает поворот
   }
   while (angle1 < 125)                //  Возвращаем сервопривод нижнего шарнира манипулятора в начальное положение
   {
    angle1++;    //  Уменьшаем угол поворота Серво1 на 1 град.
    servo1.write(angle1);    //  Сообщаем Серво1 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво1 не сделает поворот
   }
   while (angle3 > 140)                //  Возвращаем сервопривод шарнира магнитного захвата в начальное положение
   {
    angle3--;   //  Уменьшаем угол поворота Серво3 на 1 град.
    servo3.write(angle3);    //  Сообщаем Серво3 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво3 не сделает поворот
   }
   while (angle2 < 170)                //  Возвращаем сервопривод среднего шарнира манипулятора в начальное положение
   {
    angle2++;     //  Уменьшаем угол поворота Серво2 на 1 град.
    servo2.write(angle2);    //  Сообщаем Серво2 угол поворота
    delay(20);    //  Ждем 20 мс, пока Серво2 не сделает поворот
   }
   digitalWrite (Led2, LOW);           //  Синий светодиод гаснет
  } else                               //  Условие, когда цвет объекта не красный и не синий
  {
   digitalWrite (Led1, LOW);           //  Красный светодиод не горит
   digitalWrite (Magnet, LOW);         //  Электромагнит выключен
   digitalWrite (Led2, LOW);           //  Синий светодиод не горит
  }

  if (cargoflag == true)               // Условие работы электромагнита
  { 
   digitalWrite (Led1, HIGH);          //  Красный светодиод горит
   digitalWrite (Magnet, HIGH);        //  Электромагнит включен
  } else                               //  Иначе электромагнит выключен
  {  
   digitalWrite (Led1, LOW);           //  Красный светодиод не горит
   digitalWrite (Magnet, LOW);         //  Электромагнит выключен
  }
}
